Orbit::Application.routes.draw do

  mount Ckeditor::Engine => '/ckeditor'

  resources :sessions

  get "logout", to: "sessions#destroy", as: "logout"


  get "/pages/edit_view" => "pages#edit_view"
  get "/pages/preview" => "pages#preview"
  get "/pages/get_categories" => "pages#get_categories"
  post "/pages/update_item_position" => "pages#update_item_position"

  get "/page_parts/getwidgets" => "page_parts#getwidgets"
  get "/page_parts/info" => "page_parts#info"
  get "/page_parts/edit_sub_part" => "page_parts#edit_sub_part"

  resources :pages
  resources :page_parts do 
      member do
        patch 'update_sub_part'
      end
    end
 
  get "/module/:name" => "home#index"
  get "/module/:name/show" => "home#show"
  # The priority is based upon order of creation: first created -> highest priority.
  # See how all your routes lay out with "rake routes".

  # You can have the root of your site routed with "root"
  root 'pages#home'
  get 'admin/authorizations/modal_select/:module(/:type(/:id))' => 'admin/authorizations#modal_select', :as => :modal_select_authorizations
  match 'admin/authorizations/add_users/:module(/:type(/:id))' => 'admin/authorizations#add_users', :as => :add_users_authorizations, via: [:get, :post]
  match 'admin/authorizations/add_roles/:module(/:type(/:id))' => 'admin/authorizations#add_roles', :as => :add_roles_authorizations, via: [:get, :post]
  match 'admin/authorizations/remove_users/:module(/:type(/:id))' => 'admin/authorizations#remove_users', :as => :remove_users_authorizations, via: [:delete]

  locales = Site.first.in_use_locales rescue I18n.available_locales

  scope "(:locale)", locale: Regexp.new((locales+[:zh_cn]).join("|")) do
    
    get 'users/form' => "users#form_page"
    get 'users/role_page' => "users#role_page"
    get 'users/check_availability' => "users#check_availability"
    get 'users/confirmation' => "users#confirmation"
    get 'users/get_role_statuses' => "users#get_role_statuses"
    patch 'users/role_update' => "users#role_update"
    patch 'users/basic_info_update' => "users#basic_info_update"

    resources :users do
      collection do
        get 'confirm_user'
        get 'post_confirmation'
      end
    end

   


    # Facebook routes

    get "/facebook/register_fb" => "facebook#register_fb"
    get "/facebook/get_friends" => "facebook#get_friends"
    get "/facebook/get_feed" => "facebook#get_feed"
    get "/facebook/disconnect" => "facebook#disconnect"
    get "/facebook/server" => "facebook#server_redirect"
    get "/facebook/profile_import" => "facebook#profile_import"

    resources :passwords
    resources :members do
      collection do
        get ':name-:uid', to: 'members#show', as: :display   
      end
    end

    namespace :admin do
      resources :dashboards do
        collection do
          get 'reload_all_content'
          get 'reload_most_visited'
          get 'reload_recent_update'
          get 'get_cpu_usage'
          get 'get_mem_usage'
          get 'get_disk_usage'
          get 'get_month_traffic'
        end
      end
      
      get "members/unapproved_members" => "members#unapproved_members"      
      resources :members do
        collection do
          get 'edit_order_card'
          get 'edit_order_list'
          post 'update_order_card'
          post 'update_order_list'
        end
        match "setting_account" => "members#setting_account" ,:as => :setting_account, via: [:get, :post]
        match "edit_passwd" => "members#edit_passwd" ,:as => :edit_passwd, via: [:get, :post]
        match "accept_member" => "members#accept_member" ,:as => :accept_member, via: [:get, :post]
        match "edit_privilege" => "members#edit_privilege" ,:as => :edit_privilege, via: [:get, :post]
      end

      get 'members/:name-:uid/:show_plugin_profile' => "members#show", :as=> :show_plugin_profile

      resources :personal_plugins

      resources :roles do
        get 'role_field'
        get 'add_attribute_field'
        post 'toggle'
      end

      resources :member_infos

      resources :role_statuses do
        post 'toggle'
      end

      get 'select_emails' => 'member_selects#select_emails', :as => "select_emails"
      
      resources :module_apps do
        resources :categories do
          collection do
            post 'create_in_form'
          end
          member do
            post 'toggle'
          end
        end
        resources :tags do
          collection do
            post 'add_to_default'
            post 'add_from_default'
            post 'create_in_form'
            post 'delete_tags'
            post 'merge'
            post 'remove_default'
            post 'update_included_default'
          end
        end
      end

      resources :assets do   
        collection do
          post 'file_upload'
          get 'delete'
          post 'delete_files'
        end
      end

      get 'authorizations(/:module(/:type(/:id)))' => 'authorizations#index', :as => :authorizations
      
      resources :authorizations
      resources :items
      resources :categories do
        member do
          post 'toggle'
        end
      end
      resources :tags do
        collection do
          post 'add_to_default'
          post 'add_from_default'
          post 'delete_tags'
          post 'merge'
          post 'remove_default'
          post 'update_included_default'
        end
      end

        resources :sites do
	       get 'mail_setting'
	       get 'site_info'
	       get 'responsive_setting'
	       get 'search_engine'
	       get 'sitemap'
	       get 'sitemap_frontend'
	       get 'sitemap_toggle', :on => :member
	       get 'system_info'
	       get 'ui_theme'
	       get 'change_design'
	       get 'reset_default_locale'
	       get 'preference'
	       get 'update_manager'
	       get 'get_update_history'
	       get 'check_updates'
	       get 'update_orbit'
	       get 'restart_server'
              get 'delete_mail_log'             
	     end
    
      resources :designs do
        collection do
          get 'upload_package'
          get 'delete'
          post 'upload_package'
        end
        member do
          post 'edit_file'  => 'designs#edit_file'
          post 'update_file' => 'designs#update_file'
          post 'upload_image' => 'designs#upload_image'
        end
      end
      get 'design_list' => 'designs#design_list'
      get 'module_store' => 'module_store#index'
      get 'module_store/show' => 'module_store#show'
      get 'module_store/download' => 'module_store#download'
      get 'module_store/remove_module' => 'module_store#remove_module'
      get 'module_store/toggle_module' => 'module_store#toggle_module'
      get 'module_store/restart_server' => 'module_store#restart_server'


      get "import" => "import#index"
      get "import/check_url" => "import#check_url"
      get "import/module_categories" => "import#module_categories"
      get "import/module_data_list" => "import#module_data_list"
      get "import/module_data" => "import#module_data"
      post "import/rss2_news" => "import#rss2_news"
      post "import/rss2_archive" => "import#rss2_archive"
      post "import/rss2_pages" => "import#rss2_pages"
      get "import/get_import_status" => "import#get_import_status"
      post "import/rss2_galleries" => "import#rss2_galleries"
      post "import/rss2_links" => "import#rss2_links"
    end

    get 'mobile', to: 'pages#home'
    get 'mobile/:page(/:page)(/:page)(/:page)', to: 'pages#show', constraints: KeywordConstraint.new

    get ':page(/:page)(/:page)(/:page)', to: 'pages#show', constraints: KeywordConstraint.new
    resources :pages
  end
  # Example of regular route:
  #   get 'products/:id' => 'catalog#view'

  # Example of named route that can be invoked with purchase_url(id: product.id)
  #   get 'products/:id/purchase' => 'catalog#purchase', as: :purchase

  # Example resource route (maps HTTP verbs to controller actions automatically):
  #   resources :products

  # Example resource route with options:
  #   resources :products do
  #     member do
  #       get 'short'
  #       post 'toggle'
  #     end
  #
  #     collection do
  #       get 'sold'
  #     end
  #   end

  # Example resource route with sub-resources:
  #   resources :products do
  #     resources :comments, :sales
  #     resource :seller
  #   end

  # Example resource route with more complex sub-resources:
  #   resources :products do
  #     resources :comments
  #     resources :sales do
  #       get 'recent', on: :collection
  #     end
  #   end

  # Example resource route with concerns:
  #   concern :toggleable do
  #     post 'toggle'
  #   end
  #   resources :posts, concerns: :toggleable
  #   resources :photos, concerns: :toggleable

  # Example resource route within a namespace:
    namespace :admin do
      # Directs /admin/products/* to Admin::ProductsController
      # (app/controllers/admin/products_controller.rb)
      resources :users
    end
end