module OrbitTag
  module Taggable

    def self.included(base)
      base.has_many :taggings, as: :taggable, autosave: true, dependent: :destroy
      base.accepts_nested_attributes_for :taggings, allow_destroy: true
      base.extend(ClassMethods)
    end

    module ClassMethods
      def with_tags(tags=[])
        tags = [tags].flatten.uniq
        result = []
        if tags.blank?
          self.all
        else
          self.all.each do |taggable|
            result << taggable if (taggable.taggings.map{|t| t.tag.id.to_s } & tags).size > 0
          end
          self.where(:id.in=>result.map{|taggable| taggable.id})
        end
      end
    end

    def tags
      self.taggings.blank? ? [] : self.taggings.map{|t| t.tag}.compact
    end

    def tags=(tags)
      tags = [tags].flatten.uniq
      Tagging.where(:taggable=>self.id).destroy_all
      tags.each do |tag|
        self.taggings.build(tag: tag)
      end
    end

    def tag_ids
      self.taggings.blank? ? [] : self.taggings.map{|t| t.tag.id}.compact
    end

  end
end